/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/colordict.h>

#include <InterViews/display.h>
#include <InterViews/session.h>

#include <OS/list.h>
#include <OS/string.h>

class ColorDictEntry
{
	public:
		ColorDictEntry();
		ColorDictEntry(const ColorDictEntry &);
		ColorDictEntry(const Color *col,const char *name);
		~ColorDictEntry();

		ColorDictEntry &operator=(const ColorDictEntry &);
		boolean operator==(const char *s) {return _name==String(s);}
		boolean operator==(const String &s) {return _name==s;}
		boolean operator!=(const char *s) {return _name!=String(s);}
		boolean operator!=(const String &s) {return _name!=s;}

		const char *name() {return _name.string();}
		const Color *color() {return _color;}
	private:
		const Color *_color;
		CopyString _name;
};

ColorDictEntry::ColorDictEntry()
{
	_color=nil;
}

ColorDictEntry::ColorDictEntry(const ColorDictEntry &c) : _name(c._name)
{
	_color=c._color;
	Resource::ref(_color);
}

ColorDictEntry::ColorDictEntry(const Color *col,const char *name) : _name(name)
{
	_color=col;
	Resource::ref(_color);
}

ColorDictEntry::~ColorDictEntry()
{
	Resource::unref(_color);
}

ColorDictEntry &ColorDictEntry::operator=(const ColorDictEntry &c)
{
	Resource::ref(c._color);
	Resource::unref(_color);
	_color=c._color;
	_name=c._name;
}

declareList(ColorList,ColorDictEntry)
implementList(ColorList,ColorDictEntry)

ColorList *ColorDictionary::_colors=nil;
long ColorDictionary::_ref=0;
static ColorDictionary init_table;

ColorDictionary::ColorDictionary()
{
	if(_colors==nil)
		_colors=new ColorList(128);
	_ref++;
}

ColorDictionary::~ColorDictionary()
{
	_ref--;
	if(_ref==0)
	{
		delete _colors;
	}
}

const Color *ColorDictionary::color(ColorIntensity r,ColorIntensity g,ColorIntensity b,float alpha,const char *name)
{
	const Color *fw;
	Color *c=new Color(r,g,b,alpha,Color::Copy);

	fw=ColorDictionary::find(*c); // Search for color values
	if(fw!=nil)
	{
		delete c;
		if(name!=nil && *name!=0 && ColorDictionary::find(name)==nil)   // symbolic name not in table
			_colors->append(ColorDictEntry(fw,name)); // make double entry
		return fw;
	}
	fw=ColorDictionary::find(name); // Search for symbolic name in table
	if(fw!=nil)
	{
		delete c;
		return fw;
	}
	fw=Color::lookup(Session::instance()->default_display(),name); // Search for s.n. in X-table
	if(fw!=nil)
	{
		delete c;
		return fw;
	}
	_colors->append(ColorDictEntry(c,name)); // Nowhere
	return c;
}

const Color *ColorDictionary::color(const char *s)
{
	return ColorDictionary::color(String(s));
}

const Color *ColorDictionary::color(const String &s)
{
	const char *str=s.string();
	const Color *f=ColorDictionary::find(str);
	const Color *c;
	
	if(f!=nil)
		return f;
	c=Color::lookup(Session::instance()->default_display(),s);
	if(c!=nil && str!=nil && *str!=0)
		_colors->append(ColorDictEntry(c,str));
	return c;
}

const Color *ColorDictionary::find(const char *str)
{
	long i,n=_colors->count();
	String s(str);

	for(i=0;i<n;i++)
		if(_colors->item_ref(i)==s)
			return _colors->item_ref(i).color();
	return nil;
}

const Color *ColorDictionary::find(const Color &c)
{
	Display *d=Session::instance()->default_display();
	long i,n=_colors->count();

	for(i=0;i<n;i++)
	{
		ColorDictEntry cde=_colors->item(i);

		if(!c.distinguished(d,cde.color()))
			return cde.color();
	}
	return nil;
}

