/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/connector.h>
#include <Graphic/glyphviewer.h>

#include <InterViews/transformer.h>

//
// connectors - for dependencies between graphics
//

Connector::~Connector()
{
}

ClassId Connector::get_classid() const
{
	return CONNECTOR_GRAPHIC;
}

boolean Connector::is_a(ClassId id) const
{
	return id==CONNECTOR_GRAPHIC || Graphic::is_a(id);
}

//

Connector::Connector(Graphic *gr): Graphic(gr)
{
}

//
//
//

implementPtrList(FlexConnList,FlexibleConnector)

static const Coord conn_size=3.0;

FixedConnector::FixedConnector(const Brush *b,const Color *s,const Color *f,Transformer *t): Connector(nil)
{
	add_point(-conn_size,-conn_size);
	add_point(-conn_size,conn_size);
	add_point(conn_size,conn_size);
	add_point(conn_size,-conn_size);
	brush(b);
	fill(f);
	stroke(s);
	if(t!=nil)
		*transformer()=*t;
	closed(true);
	_flex_list=new FlexConnList;
}

FixedConnector::~FixedConnector()
{
	disconnect_all();
	delete _flex_list;
}

void FixedConnector::connect(Connector *c)
{
	long l,count=_flex_list->count();

	if(c->is_a(FLEXCONN_GRAPHIC))
	{
		FlexibleConnector *fc=(FlexibleConnector *) c;

		for(l=0;l<count;l++)
			if(_flex_list->item(l)==fc)
				return;
		_flex_list->append(fc);
		c->connect(this);
	}
}

void FixedConnector::disconnect(GlyphIndex ci)
{
	long i,count;
	FlexibleConnector *c=_flex_list->item(ci);

	_flex_list->remove(ci);
	((Connector *) c)->disconnect(this);
}

void FixedConnector::disconnect_all()
{
	long i,count=_flex_list->count();

	for(i=count-1;i>=0;i--)
		disconnect(i);
}

GlyphIndex FixedConnector::c_count() const
{
	return _flex_list->count();
}

Connector *FixedConnector::connector(GlyphIndex ci) const
{
	return _flex_list->item(ci);
}

Glyph *FixedConnector::clone() const
{
	return new FixedConnector(_brush,_stroke,_fill,_t);
}

void FixedConnector::update(GlyphViewer *gv)
{
	GlyphIndex l,count=_flex_list->count();

	for(l=0;l<count;l++)
		_flex_list->item(l)->update(gv);
	Connector::update();
}

ClassId FixedConnector::get_classid() const
{
	return FIXEDCONN_GRAPHIC;
}

boolean FixedConnector::is_a(ClassId id) const
{
	return id==FIXEDCONN_GRAPHIC || Connector::is_a(id);
}

//

FixedConnector::FixedConnector(Graphic *g): Connector(g)
{
	_flex_list=new FlexConnList;
}

void FixedConnector::disconnect(Connector *c)
{
	GlyphIndex l,count=_flex_list->count();

	for(l=0;l<count;l++)
		if((FlexibleConnector *) c==_flex_list->item(l))
		{
			_flex_list->remove(l);
			return;
		}
}

//
//
//

FlexibleConnector::FlexibleConnector(const Brush *b,const Color *s,const Color *f,Transformer *t): Connector(nil)
{
	add_point(-conn_size,-conn_size);
	add_point(-conn_size,conn_size);
	add_point(conn_size,conn_size);
	add_point(conn_size,-conn_size);
	brush(b);
	fill(f);
	stroke(s);
	if(t!=nil)
		*transformer()=*t;
	closed(true);
	_fixed=nil;
}

FlexibleConnector::~FlexibleConnector()
{
	disconnect(_fixed);
}

void FlexibleConnector::connect(Connector *c)
{
	if(c->is_a(FIXEDCONN_GRAPHIC))
	{
		FixedConnector *fc=(FixedConnector *) c;

		if(fc!=_fixed)
		{
			if(_fixed!=nil)
				((Connector *) _fixed)->disconnect(this);
			_fixed=fc;
			_fixed->connect(this);
		}
	}
}

void FlexibleConnector::disconnect(GlyphIndex ci)
{
	if(ci==0 && _fixed!=nil)
	{
		((Connector *) _fixed)->disconnect(this);
		_fixed=nil;
	}
}

void FlexibleConnector::disconnect_all()
{
	disconnect(GlyphIndex(0));
}

GlyphIndex FlexibleConnector::c_count() const
{
	return _fixed!=nil;
}

Connector *FlexibleConnector::connector(GlyphIndex ci) const
{
	return ci==0 ? _fixed : nil;
}

Glyph *FlexibleConnector::clone() const
{
	return new FlexibleConnector(_brush,_stroke,_fill,_t);
}

void FlexibleConnector::update(GlyphViewer *gv)
{
	if(_fixed!=nil)
	{
		Coord x1,y1,x2,y2;

		_fixed->getcenter(x2,y2);
		getcenter(x1,y1);
		if(x1!=x2 || y1!=y2)
		{
			Transformer *tm=root()->transformer();
			tm->inverse_transform(x1,y1);
			tm->inverse_transform(x2,y2);
			x2-=x1;
			y2-=y1;
			if(gv!=nil)
			{
				Coord l,b,r,t;

				getbounds(l,b,r,t);
				gv->damage(l,b,r,t);
				translate(x2,y2);
				getbounds(l,b,r,t);
				gv->damage(l,b,r,t);
			}
			else
				translate(x2,y2);
		}
		getcenter(x2,y2);
	}
	Connector::update();
}

ClassId FlexibleConnector::get_classid() const
{
	return FLEXCONN_GRAPHIC;
}

boolean FlexibleConnector::is_a(ClassId id) const
{
	return id==FLEXCONN_GRAPHIC || Connector::is_a(id);
}

//

FlexibleConnector::FlexibleConnector(Graphic *g): Connector(g)
{
	_fixed=nil;
}

void FlexibleConnector::disconnect(Connector *c)
{
	if(c==_fixed)
		_fixed=nil;
}

